@@ -45,13 +45,13 @@ pub struct MerkleTree {
 }
 
 #[derive(Debug, PartialEq)]
-pub struct ProofEntry<'a>(&'a Hash, Option<&'a Hash>, Option<&'a Hash>);
+pub struct ProofEntry<'a>(&'a [u8], Option<&'a [u8]>, Option<&'a [u8]>);
 
 impl<'a> ProofEntry<'a> {
     pub fn new(
         target: &'a Hash,
-        left_sibling: Option<&'a Hash>,
-        right_sibling: Option<&'a Hash>,
+        left_sibling: Option<&'a [u8]>,
+        right_sibling: Option<&'a [u8]>,
     ) -> Self {
         assert!((None == left_sibling) ^ (None == right_sibling));
         Self(target, left_sibling, right_sibling)
@@ -66,13 +66,18 @@ impl<'a> Proof<'a> {
         self.0.push(entry)
     }
 
-    pub fn verify(&self, candidate: Hash) -> bool {
-        let result = self.0.iter().try_fold(candidate, |candidate, pe| {
-            let lsib = pe.1.unwrap_or(&candidate);
-            let rsib = pe.2.unwrap_or(&candidate);
+    pub fn verify(&self, target: &[u8]) -> bool {
+        let hash = hash_leaf!(target);
+        self.verify_by_hash(hash)
+    }
+
+    pub fn verify_by_hash(&self, target_hash: Hash) -> bool {
+        let result = self.0.iter().try_fold(target_hash, |target_hash, pe| {
+            let lsib = pe.1.unwrap_or(&target_hash);
+            let rsib = pe.2.unwrap_or(&target_hash);
             let hash = hash_intermediate!(lsib, rsib);
 
-            if hash == *pe.0 { Some(hash) } else { None }
+            if &hash[..] == pe.0 { Some(hash) } else { None }
         });
         matches!(result, Some(_))
     }
@@ -160,17 +165,17 @@ impl MerkleTree {
 
     #[inline(always)]
     pub fn get_proof_path(&self, target: &[u8]) -> Option<Proof> {
-        let hash = hashv(&[target]);
-        self.get_proof_path_by_hash(&hash)
+        let hash = hash_leaf!(target);
+        self.get_proof_path_by_hash(hash)
     }
 
     #[inline(always)]
-    pub fn get_proof_path_by_hash(&self, target_hash: &[u8]) -> Option<Proof> {
-        let idx = self.hash_to_idx.get(target_hash)? as usize;
-        self.get_proof_path_by_idx(idx)
+    pub fn get_proof_path_by_hash(&self, target_hash: Hash) -> Option<Proof> {
+        let idx = self.hash_to_idx.get(&target_hash)? as usize;
+        self.get_proof_path_by_index(idx)
     }
 
-    pub fn get_proof_path_by_idx(&self, index: usize) -> Option<Proof> {
+    pub fn get_proof_path_by_index(&self, index: usize) -> Option<Proof> {
         if index >= self.leaf_count {
             return None;
         }
@@ -269,33 +274,44 @@ mod tests {
     fn test_path_creation() {
         let mt = MerkleTree::new(TEST);
         for (i, _s) in TEST.iter().enumerate() {
-            let _path = mt.get_proof_path_by_idx(i).unwrap();
+            let _path = mt.get_proof_path_by_index(i).unwrap();
         }
     }
 
     #[test]
     fn test_path_creation_bad_index() {
         let mt = MerkleTree::new(TEST);
-        assert_eq!(mt.get_proof_path_by_idx(TEST.len()), None);
+        assert_eq!(mt.get_proof_path_by_index(TEST.len()), None);
     }
 
     #[test]
-    fn test_path_verify_good() {
+    fn test_path_verify_by_hash_good() {
         let mt = MerkleTree::new(TEST);
+
+        for s in TEST.iter() {
+            let path = mt.get_proof_path(s).unwrap();
+            assert!(path.verify(s));
+        }
+
         for (i, s) in TEST.iter().enumerate() {
             let hash = hash_leaf!(s);
-            let path = mt.get_proof_path_by_idx(i).unwrap();
-            assert!(path.verify(hash));
+            let path = mt.get_proof_path_by_index(i).unwrap();
+            assert!(path.verify_by_hash(hash));
         }
     }
 
     #[test]
-    fn test_path_verify_bad() {
+    fn test_path_verify_by_hash_bad() {
         let mt = MerkleTree::new(TEST);
+
+        for s in BAD.iter() {
+            assert!(mt.get_proof_path(s).is_none());
+        }
+
         for (i, s) in BAD.iter().enumerate() {
             let hash = hash_leaf!(s);
-            let path = mt.get_proof_path_by_idx(i).unwrap();
-            assert!(!path.verify(hash));
+            let path = mt.get_proof_path_by_index(i).unwrap();
+            assert!(!path.verify_by_hash(hash));
         }
     }
 
