@@ -4,7 +4,7 @@
 
 use crate::{
     versioned::mapx_ord_rawkey::{MapxOrdRawKeyVs, MapxOrdRawKeyVsIter},
-    BranchName, ParentBranchName, ValueEnDe, VersionName,
+    BranchName, ParentBranchName, ValueEnDe, VersionName, VsMgmt,
 };
 use ruc::*;
 use serde::{Deserialize, Serialize};
@@ -56,29 +56,29 @@ impl<T: ValueEnDe> VecxVs<T> {
     }
 
     #[inline(always)]
-    pub fn push(&mut self, v: T) {
+    pub fn push(&self, v: T) {
         self.push_ref(&v)
     }
 
     #[inline(always)]
-    pub fn push_ref(&mut self, v: &T) {
+    pub fn push_ref(&self, v: &T) {
         self.inner
             .insert_ref(&(self.len() as u64).to_be_bytes(), v)
             .unwrap();
     }
 
     #[inline(always)]
-    pub fn pop(&mut self) -> Result<Option<T>> {
+    pub fn pop(&self) -> Result<Option<T>> {
         alt!(self.is_empty(), return Ok(None));
         self.inner.remove(&(self.len() - 1).to_be_bytes()).c(d!())
     }
 
-    pub fn update(&mut self, idx: usize, v: T) -> Result<Option<T>> {
+    pub fn update(&self, idx: usize, v: T) -> Result<Option<T>> {
         self.update_ref(idx, &v).c(d!())
     }
 
     #[inline(always)]
-    pub fn update_ref(&mut self, idx: usize, v: &T) -> Result<Option<T>> {
+    pub fn update_ref(&self, idx: usize, v: &T) -> Result<Option<T>> {
         if idx < self.len() {
             self.inner
                 .insert_ref(&(idx as u64).to_be_bytes(), v)
@@ -127,19 +127,19 @@ impl<T: ValueEnDe> VecxVs<T> {
     }
 
     #[inline(always)]
-    pub fn push_by_branch(&mut self, v: T, branch_name: BranchName) {
+    pub fn push_by_branch(&self, v: T, branch_name: BranchName) {
         self.push_ref_by_branch(&v, branch_name)
     }
 
     #[inline(always)]
-    pub fn push_ref_by_branch(&mut self, v: &T, branch_name: BranchName) {
+    pub fn push_ref_by_branch(&self, v: &T, branch_name: BranchName) {
         self.inner
             .insert_ref_by_branch(&(self.len() as u64).to_be_bytes(), v, branch_name)
             .unwrap();
     }
 
     #[inline(always)]
-    pub fn pop_by_branch(&mut self, branch_name: BranchName) -> Result<Option<T>> {
+    pub fn pop_by_branch(&self, branch_name: BranchName) -> Result<Option<T>> {
         alt!(self.is_empty(), return Ok(None));
         self.inner
             .remove_by_branch(&(self.len() - 1).to_be_bytes(), branch_name)
@@ -147,7 +147,7 @@ impl<T: ValueEnDe> VecxVs<T> {
     }
 
     pub fn update_by_branch(
-        &mut self,
+        &self,
         idx: usize,
         v: T,
         branch_name: BranchName,
@@ -157,7 +157,7 @@ impl<T: ValueEnDe> VecxVs<T> {
 
     #[inline(always)]
     pub fn update_ref_by_branch(
-        &mut self,
+        &self,
         idx: usize,
         v: &T,
         branch_name: BranchName,
@@ -239,7 +239,9 @@ impl<T: ValueEnDe> VecxVs<T> {
             iter: self.inner.iter_by_branch_version(branch_name, version_name),
         }
     }
+}
 
+impl<T: ValueEnDe> VsMgmt for VecxVs<T> {
     crate::impl_vs_methods!();
 }
 
